@@ -9,7 +9,7 @@ from flask_openapi3.models import Info, Tag
 info = Info(title='book API', version='1.0.0')
 app = OpenAPI(__name__, info=info)
 
-book_tag = Tag(name='book', description='图书')
+book_tag = Tag(name='book', description='Some Book')
 
 
 class BookData(BaseModel):
