@@ -1,19 +1,24 @@
 # -*- coding: utf-8 -*-
 # @Author  : llc
 # @Time    : 2021/5/1 21:34
-import inspect
 
+import inspect
+from http import HTTPStatus
+from typing import Dict, Type, Callable, List, Tuple, Any
+
+from flask import Response as _Response
 from pydantic import BaseModel
 from werkzeug.routing import parse_rule
 
-from .models import OPENAPI3_REF_TEMPLATE, OPENAPI3_REF_PREFIX
+from .models import OPENAPI3_REF_TEMPLATE, OPENAPI3_REF_PREFIX, Tag
 from .models.common import Schema, Response, MediaType
 from .models.parameter import ParameterInType, Parameter
 from .models.path import Operation, RequestBody, PathItem
 from .models.validation_error import UnprocessableEntity
+from .status import HTTP_STATUS
 
 
-def _parse_rule(rule):
+def _parse_rule(rule: str) -> str:
     """parse route"""
     uri = ''
     for converter, args, variable in parse_rule(str(rule)):
@@ -24,31 +29,33 @@ def _parse_rule(rule):
     return uri
 
 
-def get_operation(func):
+def get_operation(func: Callable) -> Operation:
     # get func documents
     doc = inspect.getdoc(func) or ''
+    doc = doc.strip()
+    lines = doc.split('\n')
     operation = Operation(
-        summary=doc.split('\n')[0],
-        description=doc.split('\n')[-1]
+        summary=lines[0] or None,
+        description=lines[0] if len(lines) == 0 else '</br>'.join(lines[1:]) or None
     )
 
     return operation
 
 
-def get_func_parameters(func, arg_name='path'):
+def get_func_parameter(func: Callable, arg_name='path') -> Type[BaseModel]:
     """get func parameters"""
     signature = inspect.signature(func)
-    return signature.parameters.get(arg_name)
+    p = signature.parameters.get(arg_name)
+    return p.annotation if p else None
 
 
-def get_schema(obj):
-    obj = obj.annotation
+def get_schema(obj: Type[BaseModel]) -> dict:
     assert inspect.isclass(obj) and \
            issubclass(obj, BaseModel), f"{obj} is invalid `pydantic.BaseModel`"
     return obj.schema(ref_template=OPENAPI3_REF_TEMPLATE)
 
 
-def parse_header(header):
+def parse_header(header: Type[BaseModel]) -> List[Parameter]:
     """parse args(header)"""
     schema = get_schema(header)
     parameters = []
@@ -68,7 +75,7 @@ def parse_header(header):
     return parameters
 
 
-def parse_cookie(cookie):
+def parse_cookie(cookie: Type[BaseModel]) -> List[Parameter]:
     """parse args(cookie)"""
     schema = get_schema(cookie)
     parameters = []
@@ -88,7 +95,7 @@ def parse_cookie(cookie):
     return parameters
 
 
-def parse_path(path):
+def parse_path(path: Type[BaseModel]) -> List[Parameter]:
     """parse args(path)"""
     schema = get_schema(path)
     parameters = []
@@ -108,7 +115,7 @@ def parse_path(path):
     return parameters
 
 
-def parse_query(query):
+def parse_query(query: Type[BaseModel]) -> Tuple[List[Parameter], dict]:
     schema = get_schema(query)
     parameters = []
     components_schemas = dict()
@@ -131,7 +138,7 @@ def parse_query(query):
     return parameters, components_schemas
 
 
-def parse_form(form):
+def parse_form(form: Type[BaseModel]) -> Tuple[Dict[str, MediaType], dict]:
     schema = get_schema(form)
     content = None
     components_schemas = dict()
@@ -158,7 +165,7 @@ def parse_form(form):
     return content, components_schemas
 
 
-def parse_body(body):
+def parse_body(body: Type[BaseModel]) -> Tuple[Dict[str, MediaType], dict]:
     schema = get_schema(body)
     content = None
     components_schemas = dict()
@@ -185,17 +192,19 @@ def parse_body(body):
     return content, components_schemas
 
 
-def get_responses(responses: dict, components_schemas, operation):
+def get_responses(responses: dict, components_schemas: dict, operation: Operation) -> None:
     """
     :param responses: Dict[str, BaseModel]
     :param components_schemas: `models.component.py` Components.schemas
     :param operation: `models.path.py` Operation
     """
+    if responses is None:
+        responses = {}
     _responses = {}
     _schemas = {}
     if not responses.get("422"):
         _responses["422"] = Response(
-            description="HTTP Validation error",
+            description=HTTP_STATUS["422"],
             content={
                 "application/json": MediaType(
                     **{
@@ -211,13 +220,13 @@ def get_responses(responses: dict, components_schemas, operation):
         )
         _schemas[UnprocessableEntity.__name__] = Schema(**UnprocessableEntity.schema())
     if not responses.get("500"):
-        _responses["500"] = Response(description='Server error')
+        _responses["500"] = Response(description=HTTP_STATUS["500"])
     for key, response in responses.items():
         assert inspect.isclass(response) and \
                issubclass(response, BaseModel), f" {response} is invalid `pydantic.BaseModel`"
         schema = response.schema(ref_template=OPENAPI3_REF_TEMPLATE)
         _responses[key] = Response(
-            description="Success",
+            description=HTTP_STATUS.get(key, ""),
             content={
                 "application/json": MediaType(
                     **{
@@ -240,30 +249,47 @@ def get_responses(responses: dict, components_schemas, operation):
     operation.responses = _responses
 
 
-def validate_responses(responses):
-    if responses is None:
-        responses = {}
+def validate_responses(responses: Dict[str, Type[BaseModel]]) -> None:
     assert isinstance(responses, dict), "invalid `dict`"
 
-    return responses
+
+def validate_response(resp: Any, responses: Dict[str, Type[BaseModel]]) -> None:
+    """validate response"""
+    print("Warning: "
+          "You are using `VALIDATE_RESPONSE=True`, "
+          "please do not use it in the production environment, "
+          "because it will reduce the performance.")
+    if isinstance(resp, tuple):  # noqa
+        _resp, status_code = resp[:2]
+    elif isinstance(resp, _Response):
+        if resp.mimetype != "application/json":
+            # only application/json
+            return
+            # raise TypeError("`Response` mimetype must be application/json.")
+        _resp, status_code = resp.json, resp.status_code  # noqa
+    else:
+        _resp, status_code = resp, 200
+    # status_code is http.HTTPStatus
+    if isinstance(status_code, HTTPStatus):
+        status_code = status_code.value
+
+    resp_model = responses.get(str(status_code))
+    if resp_model is None:
+        return
+    assert inspect.isclass(resp_model) and \
+           issubclass(resp_model, BaseModel), f"{resp_model} is invalid `pydantic.BaseModel`"
+    try:
+        resp_model(**_resp)
+    except TypeError:
+        raise TypeError(f"`{resp_model.__name__}` validation failed, must be a mapping.")
 
 
-def validate_response(resp, responses):
-    """validate response(only validate 200)"""
-    if responses is None:
-        responses = {}
-    for key, response in responses.items():
-        if key != "200":
-            continue
-        assert inspect.isclass(response) and \
-               issubclass(response, BaseModel), f"{response} is invalid `pydantic.BaseModel`"
-        _resp = resp
-        if isinstance(resp, tuple):  # noqa
-            _resp = resp[0]
-        response(**_resp)
-
-
-def parse_and_store_tags(new_tags, old_tags, old_tag_names, operation):
+def parse_and_store_tags(
+        new_tags: List[Tag] = None,
+        old_tags: List[Tag] = None,
+        old_tag_names: List[str] = None,
+        operation: Operation = None
+) -> None:
     """store tags
     :param new_tags: api tag
     :param old_tags: openapi doc tags
@@ -276,22 +302,30 @@ def parse_and_store_tags(new_tags, old_tags, old_tag_names, operation):
         if tag.name not in old_tag_names:
             old_tag_names.append(tag.name)
             old_tags.append(tag)
-    operation.tags = set(list([tag.name for tag in new_tags]))
+    operation.tags = list(set([tag.name for tag in new_tags])) or None
 
 
-def parse_parameters(func, components_schemas, operation):
+def parse_parameters(
+        func: Callable,
+        components_schemas: dict = None,
+        operation: Operation = None,
+        doc_ui: bool = True,
+) -> Tuple[Type[BaseModel], Type[BaseModel], Type[BaseModel], Type[BaseModel], Type[BaseModel], Type[BaseModel]]:
     """
     :param func: flask view func
     :param components_schemas: `models.component.py` Components.schemas
     :param operation: `models.path.py` Operation
+    :param doc_ui: add openapi document UI(swagger and redoc). Defaults to True.
     """
     parameters = []
-    header = get_func_parameters(func, 'header')
-    cookie = get_func_parameters(func, 'cookie')
-    path = get_func_parameters(func, 'path')
-    query = get_func_parameters(func, 'query')
-    form = get_func_parameters(func, 'form')
-    body = get_func_parameters(func, 'body')
+    header = get_func_parameter(func, 'header')
+    cookie = get_func_parameter(func, 'cookie')
+    path = get_func_parameter(func, 'path')
+    query = get_func_parameter(func, 'query')
+    form = get_func_parameter(func, 'form')
+    body = get_func_parameter(func, 'body')
+    if doc_ui is False:
+        return header, cookie, path, query, form, body
     if header:
         _parameters = parse_header(header)
         parameters.extend(_parameters)
@@ -326,7 +360,7 @@ def parse_parameters(func, components_schemas, operation):
     return header, cookie, path, query, form, body
 
 
-def parse_method(uri, method, paths, operation):
+def parse_method(uri: str, method: str, paths: dict, operation: Operation) -> None:
     """
     :param uri: api route path
     :param method: get post put delete patch
